#include "Main.h" 

#include "DDDisplay.h"
#include "DIKeyboard.h"
#include "DIMouse.h"
#include "Render.h"

#include <windows.h>

#define IDI_MAIN_ICON 101
#define IDR_MENU 102
#define IDM_EXIT 40001

#define NAME "Real Virtuality"

long WINAPI WindowProc(HWND hWnd, unsigned int Message, unsigned int wParam, long lParam)
{
   switch(Message)
   {
      case WM_ACTIVATE:
         // Pause if minimized
         Display.Active = !((BOOL)HIWORD(wParam));
         return 0L;

      case WM_DESTROY:
         // Clean up and close the app
			PostQuitMessage(0);
         return 0L;

      case WM_KEYDOWN:
         // hWnd any non-accelerated key commands
         switch(wParam)
         {
            case VK_ESCAPE:
            case VK_F12:
               PostMessage(hWnd, WM_CLOSE, 0, 0);
               return 0L;
         }
         break;

      case WM_SETCURSOR:
         // Turn off the cursor since this is a full-screen app
         SetCursor(NULL);
         return true;
   }

   return DefWindowProc(hWnd, Message, wParam, lParam);
}

HWND InitApp(HINSTANCE hInstance, int nCmdShow)
{
   HWND hWnd = NULL;
   WNDCLASS WC;

    // Set up and register window class
	WC.lpfnWndProc = WindowProc;   // Call back function
	WC.style = CS_HREDRAW | CS_VREDRAW;
	WC.cbClsExtra = 0;
	WC.cbWndExtra = 0;
	WC.hInstance = hInstance;
	WC.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MAIN_ICON));
	WC.hCursor = LoadCursor(NULL, IDC_ARROW);
	WC.hbrBackground = (HBRUSH )GetStockObject(BLACK_BRUSH);
	WC.lpszMenuName = NAME;
	WC.lpszClassName = NAME;
	RegisterClass(&WC);

	// Create a window
	hWnd = CreateWindowEx(WS_EX_TOPMOST, NAME, NAME, WS_POPUP, 0, 0, GetSystemMetrics(SM_CXSCREEN), GetSystemMetrics(SM_CYSCREEN), NULL, NULL, hInstance, NULL);

	ShowWindow(hWnd, nCmdShow);
   UpdateWindow(hWnd);
   SetFocus(hWnd);

   return hWnd;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
   MSG Msg;
   HWND hWnd;

   hPrevInstance;   // Just to avoid compiler warning
   lpCmdLine;   // Just to avoid compiler warning

	int Width = 1024;
	int Height = 768;
	int BPP = 32;

   hWnd = InitApp(hInstance, nCmdShow);

	Display.Init(hWnd, Width, Height, BPP);
	Input.Init(hWnd);
	Keyboard.Init();
	Keyboard.SetAcquire();
	Mouse.Init();
	Mouse.SetAcquire();

   while(true)
   {
      if(PeekMessage(&Msg, NULL, 0, 0, PM_NOREMOVE))
      {
         if (!GetMessage(&Msg, NULL, 0, 0))
            return Msg.wParam;
         TranslateMessage(&Msg);
         DispatchMessage(&Msg);
      }
      else if(Display.Active)
         Render();
      else
         WaitMessage();
   }
}
